from __future__ import annotations

import argparse
from pathlib import Path
import sys
from typing import List

import yaml

REQUIRED_ATTR_ORDER = [
    "aliases",
    "desc",
    "internal",
    "summary",
    "label",
    "prefix",
    "silent",
    "dotenv",
    "env",
    "vars",
    "requires",
    "sources",
    "dir",
    "platform",
    "set",
    "shopt",
    "generates",
    "method",
    "prompt",
    "run",
    "deps",
    "preconditions",
    "status",
    "cmds",
    "ignore_error",
    "interactive",
]


def main(args: List[str] | None = None):
    args_parser = argparse.ArgumentParser(
        description="Links the given YAML file as a Taskfile."
    )
    args_parser.add_argument("--file", help="YAML file to lint.", required=True)
    parsed_args = args_parser.parse_args(args)
    taskfile_path: Path = Path(parsed_args.file)

    with open(taskfile_path, "r") as f:
        doc = yaml.safe_load(f)

    failed = False
    for task_name, attrs in doc["tasks"].items():
        attr_names = list(attrs.keys())
        sorted_attr_names = sorted(attr_names, key=lambda x: REQUIRED_ATTR_ORDER.index(x))
        is_unsorted = False
        for a, b in zip(attr_names, sorted_attr_names):
            if a != b:
                is_unsorted = True
                failed = True
                break
        if is_unsorted:
            print(f"{taskfile_path}: Task '{task_name}' has unsorted attributes.")
            print(f"\tExpected order: {sorted_attr_names}")
            print(f"\tActual order  : {attr_names}")

    if failed:
        return 1

    return 0


if "__main__" == __name__:
    sys.exit(main())
